
if (isObject(TECMBindEditor))
	TECMBindEditor.delete();

new ScriptObject(TECMBindEditor) {

	class = TECMBindEditor;

	Version = "4.0";
	DisplayName = "ECM Bind Editor";
	

};



//Bind Editor Functions________________________________________________________________________________
//-------------------------------------------------------------------------------------------------------

	//Initialize ECM Setup Dialog (Bind Editor)
	function ECMEditMenu::onWake( %this ) {

		//Setup VoiceMode List (2 Items)
		ECMVoiceMode.clear();
		
		//Set Auto Over-ride toggle
		ECMAutoOverrideToggle.setValue($pref::TracerECM::AutoOverride);

		
		ECMVoiceMode.add("Enhanced", 0);
		//Only allow selection of NullVoice Mode when not connected
		if (!isObject(PlayerListGroup)) 
			ECMVoiceMode.add("Null Voice", 1);
		ECMVoiceMode.add("Normal", 2);
		
		//Set VoiceMode list
		ECMVoiceMode.setSelected($pref::TracerECM::OperationalMode);

		//Load Menu File (Based on Current Voice and and Voice Mode)
		if (!$ECMConversion::Converting)
			ECMenuBuild.activateMenu();

		//Fill GUI tree with loaded data
		TECMBindEditor.fillMenuTree();

	}

	//When Done editing.
	function EditECMMenuGui::onSleep( %this ) {


		//Clear GUI Tree
		ECMGuiTree.clear();

		//Reload Menu (Based on Current Voice and and Voice Mode)
		ECMenuBuild.activateMenu();

	}

	//Change VoiceMode Dropdown Selection...
	function ECMVoiceMode::onSelect( %this ) {


		//If Current Voice File Changed...
		if ( ECMGuiTree.dirty )
			MessageBoxYesNo( "CHANGES", "Do you want to save your changes?", "TECMBindEditor.saveMenu(); TECMBindEditor.ReallySelectMode();", "TECMBindEditor.ReallySelectMode();" );
		else
			TECMBindEditor.ReallySelectMode();
		
	}

	//Called when we "really" want to Select a voice mode
	function TECMBindEditor::ReallySelectMode( %this ) {

		//When in a server, we change the "Operational Mode", when in the lobby we change the actual "Voice Mode"
		//Set Global Pref Variable Accordingly
		if (isObject(PlayerListGroup))
			$pref::TracerECM::OperationalMode = ECMVoiceMode.getSelected();
		else
			$pref::TracerECM::VoiceMode = ECMVoiceMode.getSelected();

		//Load the new corresponding menu data file
		//TECMBindEditor.doLoadMenu(); - No longer aplicable, since all mode's data is stored in one universal tree now

	}

	//Switches menu in the Editor.
	function TECMBindEditor::doLoadMenu( %this ) {

		//Load New Voice Menu Data File (Based on Current Voice and and Voice Mode)
		if (!$ECMConversion::Converting)
			ECMenuBuild.activateMenu();
		

		//Build new GUI Tree with newly loaded data
		TECMBindEditor.fillMenuTree();

	}

	function TECMBindEditor::fillMenuTree( %this ) {

		//Reset Bind GUI Tree
		ECMGuiTree.clear();

		//Grab the first "root item" of newly cleared tree
		%guiRoot = ECMGuiTree.getFirstRootItem();

		//Check VoiceMode, add root item accordingly
		%newGuiId = ECMGuiTree.insertItem( %guiRoot, "Enhanced Chat Menu", 0 );

		//Write Data Starting at SimGroup ECMenuBuild.RootMenu to GUI Tree Item: %newGuiId
		TECMBindEditor.traverseMenu( %newGuiId, ECMenuBuild.RootMenu );

		//Expand Topmost Item in the GUI Tree
		ECMGuiTree.expandItem( %newGuiId );

		//Select Topmost Item in the GUI Tree
		ECMGuiTree.selectItem( %newGuiId );
		
		//Stor this root item for use later
		ECMGuiTree.rootMenuItem = %newGuiId;

		//Clear "Dirty" flag
		ECMGuiTree.dirty = false;
	}

	
	//Self Calling Function that "writes" the current voice menu to the GUI tree
	function TECMBindEditor::traverseMenu( %this, %guiID, %menu ) {

		//For Each Item in Current Menu Level
		for ( %i = 0; %i < %menu.optionCount; %i++ ) {

			//.option: Menu Display Text
			%text = %menu.option[%i];

			//If this item is a VoicePackDX Registered Item, Colorize it 
			//(This Color Excape is also used to prevent editing and saveing of VoicePack Items to Custom Menu Files)
			if (%menu.isVoicePack[%i])
				%text = "\c4" @ %text;

			//isMenu: Boolean for either a submenu item or command item
			if ( %menu.isMenu[%i] ) {

				//Create a Menu Item in the GUI Tree
				%newGuiID = ECMGuiTree.insertItem( %guiID, %text, 0 );
				TECMBindEditor.traverseMenu( %newGuiID, %menu.command[%i] );

			}
			else {

				//Create a Command Item in the GUI Tree.
				%newGuiID = ECMGuiTree.insertItem( %guiID, %text, 1 );
				TECMBindEditor.commandObj[%newGuiID] = %menu.command[%i];

			}

		}

	}	
	
	//GUI EVENT: Handle Right Clicks on the GUI Tree
	function ECMGuiTree::onRightMouseDown( %this, %item, %pos ) {

	   // launch the action menu (ECMItemActionPopup::awaken)...
	   ECMGuiTree.selectItem( %item );
	   ECMItemActionPopup.awaken( %item, %pos );

	}

	//Called to show the right Click Menu...
	function ECMItemActionPopup::awaken( %this, %item, %pos ) {

		//Initialize GUI Panel to mouse position and clear items
		%this.position = %pos;
		%this.clear();

		//Grab GUI Tree Root Item Object
		%treeRoot = ECMGuiTree.getFirstRootItem();

		//Determine if this is a Menu or Bind Item
		%isMenu = ( strcmp( ECMGuiTree.getItemValue( %item ), "0" ) == 0 );

		//Add Edit Item if this is not the Root Item
		if ( %item != %treeRoot )
			%this.addEntry( "Edit", 0 );

		//Add AddMenu and Add Command Options if this is a menu item
		if ( %isMenu ) {
			%this.addEntry( "Add menu", 1 );
			%this.addEntry( "Add command", 2 );
		}

		//If this item has a previous sibling (Item Above it) add Move Up option
		if ( ECMGuiTree.getPrevSibling( %item ) )
			%this.addEntry( "Move up", 3 );

		//If this item has a next sibling (Item Below it) add Move Down option	
		if ( ECMGuiTree.getNextSibling( %item ) )
			%this.addEntry( "Move down", 4 );

		//If this is not the root item, add Delete option
		if ( %item != %treeRoot )
			%this.addEntry( "Delete", 5 );

		//If no items were able to be added, cancel the menu operation
		if ( %this.numEntries == 0 )
			return;

		//Show the newly constructed right click menu
		Canvas.pushDialog( ECMItemActionDlg ); 

		//Causes %this.onCancel to be called if user clicks outside of the menu
		%this.forceOnAction();

	}

	//Called during menu construction to add an option to right click menu
	function ECMItemActionPopup::addEntry( %this, %text, %id ) {

		//Increment Item Count Counter
		%this.numEntries++;

		//Add the item to the menu
		%this.add( %text, %id );

	}

	//Clear the menu from the screen
	function ECMItemActionPopup::reset( %this ) {

		//Clear the entry counter
		%this.numEntries = 0;

		//Cancels the onCancel event caused by %this.forceOnAction();
		%this.forceClose();

		//Remove the menu from visible GUI parent object
		Canvas.popDialog( ECMItemActionDlg );

	}

	//Called when the user selects and item
	function ECMItemActionPopup::onSelect( %this, %id, %text ) {

		//Grab the selected GUI Tree item
		%item = ECMGuiTree.getSelectedItem();

		//Grab the Text in the selected GUI Tree
		%temp = ECMGuiTree.getItemText( %item );

		//Check for VoicePackDX Item Escape sequence in text (\c4)
		if ((getSubStr(%temp,0,1) $= "\c4") || (getSubStr(%temp,1,1) $= "-")) {

			MessageBoxOK( "VOICE PACK", "This item is part of a custom voice pack, and cannot be edited!" );

		} else {
			switch ( %id ) {
				case 0: // Edit

					//Grab a few data items from the text
					%temp = ECMGuiTree.getItemText( %item );
					%key = getSubStr( %temp, 0, strpos(%temp, ":") );
					%text = getSubStr(%temp, (strpos(%temp, ":")+2), (strlen(%temp) - (strpos(%temp, ":")+2)));
					%command = ECMGuiTree.getItemValue( %item );

					//Call Edit Menu or Edit Bind Functions accordingly...
					if ( strcmp( %command, "0" ) == 0 )
						TECMBindEditor.editMenu( %key, %text, "TECMBindEditor.doEditMenu" );
					else {
						TECMBindEditor.editSelectedItem();
					}

				case 1: // Add menu
					TECMBindEditor.editMenu( "", "", "TECMBindEditor.doNewMenu" );         
				case 2: // Add command
					ECMCreateBlankCommandObj();
				case 3: // Move up - Built in GUI Functions make this action easy
					ECMGuiTree.moveItemUp( %item );
					ECMGuiTree.dirty = true; //Set Dirty Flag
				case 4: // Move down  - Built in GUI Functions make this action easy
					%nextItem = ECMGuiTree.getNextSibling( %item );
					ECMGuiTree.moveItemUp( %nextItem );
					ECMGuiTree.dirty = true; //Set Dirty Flag
				case 5: // Delete  - Built in GUI Functions make this action easy
					ECMGuiTree.removeItem( %item );
					ECMGuiTree.dirty = true; //Set Dirty Flag
			}
		}

		//Call Menu Removeal Function
		%this.reset();
	}

	//Called if user clicks outside of menu (caused by %this.forceOnAction();)
	function ECMItemActionPopup::onCancel( %this ) {

		//Call Menu Removeal Function
		%this.reset();

	}

	//Called when User Chooses to edit a bind item in the GUI Tree with the ItemActionPopup (rightclick menu)
	//This is kind of tricky, so it requires this first function before the actual "edit" function: editCommand
	function TECMBindEditor::editSelectedItem( %this ) {

		//Grab the selected item to be edited
		%item = ECMGuiTree.getSelectedItem();

		//Make sure we aren't trying to edit the root item (not allowed)
		if ( %item != ECMGuiTree.getFirstRootItem() ) {

			//Grab the text of the the item to be edited
			%temp = ECMGuiTree.getItemText( %item );

			//Ensure we aren't trying to edit a proteted VoicePack item
			if ((getSubStr(%temp,0,1) $= "\c4") || (getSubStr(%temp,1,1) $= "-")) {

				MessageBoxOK( "VOICE PACK", "This item is part of a custom voice pack, and cannot be edited!" );

			} else {

				//Make sure that this item isn't empty, if it is, we have a problem, TODO: Add a handle for this for stability
				if ( strlen( %temp ) > 0 ) {

					//Parse some info from the text of this item
					%key = getSubStr( %temp, 0, strpos(%temp, ":") );
					%text = getSubStr(%temp, (strpos(%temp, ":")+2), (strlen(%temp) - (strpos(%temp, ":")+2)));
					%command = ECMGuiTree.getItemValue( %item );

					//Menu or command?...
					if ( %command $= "0" )

						//Menu
						TECMBindEditor.editMenu( %key, %text, "TECMBindEditor.doEditMenu" );

					else {

						//Command
						%cmdSpot = strpos(%text, " - (");
						%cmdName = getSubStr(%text, 0, %cmdSpot);
						%cmdText = getSubStr(%text, (%cmdSpot + 4), (strlen(%text) - (strlen(%cmdName) + 5)));

						%commandObj = TECMBindEditor.commandObj[%item];

						//Now we can call the edit function
						TECMBindEditor.editCommand( %key, %text, %commandObj );

					}
				}
			}
		}
	}

	//Called when a Command Item needs to be edited
	function TECMBindEditor::editCommand( %this, %key, %text, %commandObj ) {

		//NOTE: We use the text box Text GUI Control's Variable property to set and get values entered by the user.
		//This requires a "true" variable: specifically the $ECMVars:: series. I usually do my best not to use true
		//Global variables (Makes script compatability issues non-existent), but this is neccassary to get this effect.

		//Set what command we're working with...
		$ECMVars::commandObj = %commandObj;
		$ECMVars::key = %key;
		$ECMVars::text = %text;

		//Show the Command Editor dialog (this causes the rest of the initialization)
		EditECMCommandDlg.Callback = "TECMdoEditCommand";
		Canvas.pushDialog( EditECMCommandDlg );

	}

	function TECMBindEditor::addCommand( %this, %commandObj ) {

		$ECMVars::commandObj = %commandObj;
		$ECMVars::key = "";
		$ECMVars::text = "New Command";

		EditECMCommandDlg.Callback = "TECMdoNewCommand";
		Canvas.pushDialog( EditECMCommandDlg );


	}

	//Called when a Menu Item neeeds to be edited
	function TECMBindEditor::editMenu( %this, %key, %text, %callback ) {

		//NOTE: We use the text box Text GUI Control's Variable property to set and get values entered by the user.
		//This requires a "true" variable: specifically the $ECMVars:: series. I usually do my best not to use true
		//Global variables, but this is neccassary to get this effect.

		//Setup the variables
		$ECMVars::key = %key;
		$ECMVars::text = %text;
		$ECMVars::OKCommand = %callback @ "($ECMVars::key, $ECMVars::text);";

		//Show the Menu Editor dialog (this causes the rest of the initialization)
		Canvas.pushDialog( EditECMMenuDlg );

	}
	
	//Called when a New Menu item needs to be created
	function TECMBindEditor::doNewMenu( %this, %key, %text ) {

		//If Any Parameters are ommitted, do not continue
		if ( strlen( %key ) && strlen( %text ) ) {

			//Hide edit dialog, we don't need it anymore
			Canvas.popDialog( EditECMMenuDlg );

			//Get selected item in the GUI Tree, so we know where to insert the new one
			%item = ECMGuiTree.getSelectedItem();
			%temp = ECMGuiTree.getItemText( %item );

			//Create the new entry
			%newText = %key @ ": " @ %text;

			//Add the new entry
			%newItem = ECMGuiTree.insertItem( %item, %newText, "0" );

			//Expand the newly created menu item
			ECMGuiTree.expandItem( %item );

			//Select the newly created menu item
			ECMGuiTree.selectItem( %newItem );

			//Warn the user if this item uses the same key as an adjacent (sibbling) item
			TECMBindEditor.CheckSiblings( %newItem );

			//Set the "durty' flag
			ECMGuiTree.dirty = true;

			

   		}

	}

	//Called when a New Bind (command) item GUI Tree entry needs to be created
	function TECMdoNewCommand( %key, %text, %command ) {

		%newText = %key @ ": " @ %text;

		//Grab the selected the GUI Tree item to insert at
		%item = ECMGuiTree.getSelectedItem();
		
		//Insert the new item
		%newItem = ECMGuiTree.insertItem( %item, %newText, "1" );

		//Add Command to command list indexed by new item
		TECMBindEditor.commandObj[%newItem] = %command;

		//Expand the new Item
		ECMGuiTree.expandItem( %item );

		//Select the new item
		ECMGuiTree.selectItem( %newItem );

		//Warn user if using the same key as an adjacent item
		TECMBindEditor.CheckSiblings( %newItem );

		//Mark for save on exit
		ECMGuiTree.dirty = true;
			
		
		
	}

	//This function is called to actually change the GUI Tree entry
	function TECMBindEditor::doEditMenu( %this, %key, %text ) {

		//Make sure changes were passed
		if ( strlen( %key ) && strlen( %text ) ) {

			//Hide Menu Item edit Dialog
			Canvas.popDialog( EditECMMenuDlg );

			//Grab the selected item in the GUI Tree to edit
			%item = ECMGuiTree.getSelectedItem();

			//Create the new text item
			%newText = %key @ ": " @ %text;

			//Change the selected item's text to the new text item
			ECMGuiTree.editItem( %item, %newText, "0" );

			//Warn user if there are duplicate keys being used by an adjacent item (sibbling)
			TECMBindEditor.CheckSiblings( %item );

			//Set dirty flag
			ECMGuiTree.dirty = true;

		}

	}

	
	function TECMdoEditCommand( %key, %text, %command) {
		
		%newText = %key @ ": " @ %text;

		//Find Item to be edited
		%item = ECMGuiTree.getSelectedItem();

		//Make changes
		ECMGuiTree.editItem( %item, %newText, "1" );

		//Check for existing conflicts
		TECMBindEditor.CheckSiblings( %item );

		//Mark for save on exit
		ECMGuiTree.dirty = true;

	}

	//Initialize the GUI based on variables set by initializing functions
	function EditECMCommandDlg::onWake(%this) {
		
		//Setup Canned Chat Item List
		ECMCannedItem.clear();
		ECMCannedItem.add("- Use Enhanced Chat -", 0);
		ECMCannedItem.setSelected(0);
		for(%e=1; ECMSoundExplorer.CannedList[%e]!$=""; %e++) {
			ECMCannedItem.add(ECMSoundExplorer.CannedList[%e], %e + 1);
			if($ECMVars::cannedchatitem $= ECMSoundExplorer.CannedList[%e])
				ECMCannedItem.setSelected(%e + 1);
			
		}
		
		//Setup Enhanced Sound List
		ECMSound.clear();
		ECMSound.add("- None -",0);
		if ($pref::TracerECM::VoiceMode == 1) {
			
			//echo("Nullvoice list Built");

			for(%e=1; ECMSoundExplorer.NullSoundList[%e]!$=""; %e++) {
				ECMSound.add(ECMSoundExplorer.NullSoundList[%e], %e + 1);
				if($ECMVars::sound $= ECMSoundExplorer.NullSoundFile[%e])
					ECMSound.setSelected(%e + 1);
				
			}
		} else {

			//echo("Enhacn list Built");

			for(%e=1; ECMSoundExplorer.SoundList[%e]!$=""; %e++) {
				ECMSound.add(ECMSoundExplorer.SoundList[%e], %e + 1);
				if($ECMVars::sound $= ECMSoundExplorer.SoundFile[%e])
					ECMSound.setSelected(%e + 1);
				
			}
		}
		

		//Setup Scope List (2 Items)
		ECMScope.clear();
		ECMScope.add("Global Chat", 0);
		ECMScope.add("Team Chat", 1);
		
		ECMScope.setSelected($ECMVars::team);

		//Clear Sound Explorer in case Voice Mode has changed
		ECMSoundItemList.clear();
	

	}
	
	//This function checks for entries with duplicate "keys" with the same parent
	function TECMBindEditor::CheckSiblings( %this, %item ) {

		//Set allClear Flag to true (No Duplicates)
		%allClear = true;

		//Grab the previous sibling
		%sibling = ECMGuiTree.getPrevSibling( %item );
		//Check siblings from here up...
		while ( %sibling != 0 ) {

			//Parse out the sibbling's key
			%siblingKey = getSubStr( ECMGuiTree.getItemText( %sibling ), 0, strpos(ECMGuiTree.getItemText( %sibling ),  ":") );

			//Check the key to the current edited/created item's key
			if ( %siblingKey $= $ECMVars::key ) {
				%allClear = false;
				break;
			}

			//Load next "prev" sibling
			%sibling = ECMGuiTree.getPrevSibling( %sibling );

		}

		//If clear, check sibblings from here down...
		if ( %allClear ) {

			//Grab next sibbling
			%sibling = ECMGuiTree.getNextSibling( %item );

			//Check siblings loop...
			while ( %sibling != 0 ) {

				//Parse out Sibbling's key
				%siblingKey = getSubStr( ECMGuiTree.getItemText( %sibling ), 0, strpos(ECMGuiTree.getItemText( %sibling ), ":") );

				//Check the key to the current edited/created item's key
				if ( %siblingKey $= $ECMVars::key ) {
					%allClear = false;
					break;
				}

				//Grab next "next" sibbling
				%sibling = ECMGuiTree.getNextSibling( %sibling );

			}

		}




		if ( !%allClear ) {

			//Parse out conflicting checked item's name
			if ( ECMGuiTree.getItemValue( %item ) $= "0" )
				%text1 = restWords( ECMGuiTree.getItemText( %item ) );
			else {
				%temp = ECMGuiTree.getItemText( %item );
				%text1 = getWords( %temp, 1, getWordCount( %temp ) - 5 );
			}

			//Parse out conflicting sibbling's name
			if ( ECMGuiTree.getItemValue( %sibling ) $= "0" )
				%text2 = restWords( ECMGuiTree.getItemText( %sibling ) );
			else {
				%temp = ECMGuiTree.getItemText( %sibling );
				%text2 = getWords( %temp, 1, getWordCount( %temp ) - 5 );
			}

			//Display a warning to the user about duplicate key entires
			MessageBoxOK( "WARNING", "Menu siblings \"" @ %text1 @ "\" and \"" @ %text2 @ "\" are both bound to the \'" @ %siblingKey @ "\' key!" );

		}

		return %allClear;

	}

	
	//Plays the currently selected sound (In the editor pane, not Sound Explorer)
	function TECMBindEditor::PlaySound( %this ) {

		//Determine if canned chat item is selected
		%canneditem = ECMCannedItem.getValue();
		
		if ( %canneditem !$= "- Use Enhanced Chat -" ) {
			
			//Grab the actual cannedchatitem ID from the text label
			for(%d=1; ECMSoundExplorer.CannedList[%d]!$=""; %d++)
			{
				
				if(ECMSoundExplorer.CannedList[%d]$=%canneditem)
				{
					%cmdID = ECMSoundExplorer.CannedItems[%d];
				}

			}
			
			//This is a Tribes 2 defined function, used to test sounds in the original chat item edit dialog
			//echo(%cmdID);
			testChatCommand(%cmdID);

		} else {

			//Get the value of the currently selected sound
			%b=ECMSound.getValue();

			//NullVoice stuff is a completely different animal, need a seprate handler
			if ($pref::TracerECM::VoiceMode == 1) {

				//Grab the Sound file from the NullSoundList
				//This is a crapload of String compares, but this is a client script not a server script ;)
				for(%d=1; ECMSoundExplorer.NullSoundList[%d]!$=""; %d++)
				{
					if(ECMSoundExplorer.NullSoundList[%d]$=%b)			
						%MsgNum=%d;
				}

				//temp is the sound to be played...
				%temp = (ECMSoundExplorer.NullSoundFile[%MsgNum] @ ".wav");

				//Search for `voice` - (Insert voice dir, to over-ride NULL voice)
				%pos = strpos( %temp, "`voice`");
				if (%pos != -1)
					%temp = ( getSubStr( %temp, 0, %pos ) @ "voice/" @ TracerECM.PlayerVoice @ getSubStr(%temp, (%pos + 7), strlen(%temp)) );


			} else {

				//Grab the SoundFile name from the Text Label list. (Cross-reference)
				for(%d=1; ECMSoundExplorer.SoundList[%d]!$=""; %d++)
				{
					//echo("'" @ ECMSoundExplorer.SoundList[%d] @ "' $= '" @ %b @ "' = " @ (ECMSoundExplorer.SoundList[%d]$=%b));
					if(ECMSoundExplorer.SoundList[%d]$=%b)			
						%MsgNum=%d;
				}

				//Create the sound to be played's file name
				%temp = "voice/" @ TracerECM.PlayerVoice @ "/" @ ECMSoundExplorer.SoundFile[%MsgNum] @ ".wav";		

			}

			//Play the sound file for the user's enjoyment
			alxPlay( alxCreateSource( AudioGui, %temp ) );

		}

	}

	function TECMBindEditor::saveMenu( %this ) {

		%filename = ("scripts/TDX/ECM/Menus/Current.mnu.cs");

		TECMBindEditor.doSaveMenu( %filename );

	}

	function TECMBindEditor::doSaveMenu( %this, %filename ) {

		//No FileName? Ok, someone else is calling this function wrong.
		if ( %filename $= "" )
			return;

		//Render a display label for the mode that this file was saved for
		if ($pref::TracerECM::VoiceMode == 1)
			%VoiceModeDisplay = "Null Voice"; 
		else if ($pref::TracerECM::VoiceMode == 2)
			%VoiceModeDisplay = "Canned Chat";
		else
			%VoiceModeDisplay = "Enhanced Chat";

		//Create a new file object so we can write this shit
		new fileObject( "saveFile" );
		saveFile.openForWrite( %filename );

		//Print out a little header so people know what the hell this file is
		saveFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
		saveFile.writeLine( "//                                                                            \\\\" );
		saveFile.writeLine( "//             TracerDX's Enhanced Chat - Menu Definition File                \\\\" );
		saveFile.writeLine( "//                                                                            \\\\" );
		saveFile.writeLine( "//             Written by TracerDX (tracer@planettracerdx.com)                \\\\" );
		saveFile.writeLine( "//                    http://www.planettracerdx.com                           \\\\" );
		saveFile.writeLine( "//                                                                            \\\\" );
		saveFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
		saveFile.writeLine( " " );
		saveFile.writeLine( " " );

		//Grab the root GUI Tree Item, start from
		%rootItem = ECMGuiTree.getFirstRootItem();

		//Begin writing GUI Tree contents to file
		TECMBindEditor.writeTreeNode( saveFile, ECMGuiTree.getChild( %rootItem ), "" );

		//Close File, delete file object (not the file)
		saveFile.close();
		saveFile.delete();

		//Clear "dirty" flag (we save ;) )
		ECMGuiTree.dirty = false;

		//Apply the changes to the currently loaded menu. 
		TECMBindEditor.activateMenu( "sctipts/TDXMenu" @ TracerECM.PlayerVoice);

		//Display a message box to the user to confirm Save
		MessageBoxOK( "SAVED", "Save successful. Menu Updated" );
	}

	//Self calling function that traverses the GUI Tree and saves all items to a file
	function TECMBindEditor::writeTreeNode( %this, %file, %item, %space ) {

		//Grab the text of the specified item
		%temp = ECMGuiTree.getItemText( %item );
		//echo("Saving Item #" @ %item);

		//Grab the Command Object
		%commandObj = TECMBindEditor.commandObj[%item];
		//echo(isObject(%commandObj) @ " - " @ %commandObj);

		
			
		//Parse a few items out of the text
		%key = getSubStr( %temp, 0, strpos(%temp, ":") );
		%text = getSubStr(%temp, (strpos(%temp, ":")+2), (strlen(%temp) - (strpos(%temp, ":")+2)));
		//echo(%key @ " - " @ %text);
		%command = ECMGuiTree.getItemValue( %item );

		//Find out if this is a menu item or bind (command) item 
		if ( strcmp( %command, "0" ) == 0 ) {

			//Write corresponding commands to file in order to rebuild this submenu: Start of a new menu
			%file.writeLine( %space @ "ECMenuBuild.startMenu( \"" @ %key @ ": " @ %text @ "\" );" );

			//Grab the first child item of this submenu
			%child = ECMGuiTree.getChild( %item );

			//If it is "real" begin another writeTreeNode thread to write them all out
			if ( %child )
				TECMBindEditor.writeTreeNode( %file, %child, ("\t" @ %space) );

			//Finish this submenu up.
			%file.writeLine( %space @ "ECMenuBuild.endMenu(); // " @ %text );
			%file.writeLine( " ");

		}
		else {

			//Parse out Data for this bind (command) item
			%commandObj = TECMBindEditor.commandObj[%item];

			//Write the command to recreate this bind (command) item
			if (%commandObj.isTeam)
				%file.writeLine( %space @ "ECMenuBuild.addCmdBind( \"" @ %key @ ": " @ %text @ "\", true);" );
			else
				%file.writeLine( %space @ "ECMenuBuild.addCmdBind( \"" @ %key @ ": " @ %text @ "\");" );
			//Begin Writing Command Descrpition Entries
			// 0 - Voice Bind Descriptions
			for (%i = 0; %i < 14; %i++) {
				%text = %commandObj.chatText[%i];
				%sound = %commandObj.chatSound[%i];
				if ((%text !$= "") && (%sound !$= ""))
					%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(0, " @ %i @ ", \"" @ ECMfixQuotesForSave(%text) @ "\", \"" @ %sound @ "\");" );
			}

			// 1 - NullVoice Bind Descriptions
			for (%i = 0; %i < 14; %i++) {
				%text = %commandObj.nullChatText[%i];
				%sound = %commandObj.nullChatSound[%i];
				if ((%text !$= "") && (%sound !$= ""))
					%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(1, " @ %i @ ", \"" @ ECMfixQuotesForSave(%text) @ "\", \"" @ %sound @ "\");" );
			}

			// 2 - ECM Function Descriptions
			if (%commandObj.ECMFunction !$= "") {
				%text = %commandObj.ECMFunction;
				%sound = %commandObj.ECMFunctionParameters;
				%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(2, " @ %i @ ", \"" @ ECMfixQuotesForSave(%text) @ "\", \"" @ ECMfixQuotesForSave(%sound) @ "\");" );
			}

			// 3 - Console Command
			if (%commandObj.consoleCommand !$= "") {
				%text = %commandObj.consoleCommand;
				%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(3, " @ %i @ ", \"" @ ECMfixQuotesForSave(%text) @ "\");" );
			}

			// 4 -Canned Chat Item
			if (%commandObj.cannedChat !$= "") {
				%text = %commandObj.cannedChat;
				%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(4, 0, \"" @ %text @ "\");" );
			}

			// 5 -Voice Pack Root Item
			if (%commandObj.startVoicePack !$="") {
				%text = %commandObj.startVoicePack;
				%file.writeLine( "\t" @ %space @ "ECMenuBuild.addCmdDesc(5, 0, \"" @ %text @ "\");" );
			}

			//End this Bind
			%file.writeLine( %space @ "ECMenuBuild.endCmdBind();" );
			
			

		}

		//Grab next item in this group to write
		%sibling = ECMGuiTree.getNextSibling( %item );

		//Continue the loop if there are more items to write...
		if ( %sibling != 0 )
			TECMBindEditor.writeTreeNode( %file, %sibling, %space );
			
		


	}

	//This function is called when the user attempts to leave the editor
	function TECMBindEditor::leaveMenuEditor( %this ) {

		//If the GUI Tree is dirty (not saved) Ask the user if they want to save.
		if ( ECMGuiTree.dirty )
			MessageBoxYesNo( "CHANGES", "Do you want to save your changes?", "TECMBindEditor.saveMenu(); TECMBindEditor.reallyLeaveMenuEditor();", "TECMBindEditor.reallyLeaveMenuEditor();" );
		else
			TECMBindEditor.reallyLeaveMenuEditor();

	}

	//Called when it is decided that user wants to leave now
	function TECMBindEditor::reallyLeaveMenuEditor( %this ) {

		//Reload Menu Data based on Voice and Mode
		ECMenuBuild.activateMenu();

		//Clear the GUI Tree
		ECMGuiTree.clear();
		
		//Save Auto Over-ride toggle
		$pref::TracerECM::AutoOverride = ECMAutoOverrideToggle.getValue();

		//Remove dialogs from visible GUI structure
		Canvas.popDialog( EditECMMenuGui );
		Canvas.popDialog( ECMEditMenu );

	}

	//Called when user wishes to revert to last saved copy
	function TECMBindEditor::resetMenu( %this ) {

		//TO DO: Show a warning dialog

		//Reload Data and Rebuild GUI Tree
		TECMBindEditor.doLoadMenu();

	}

	//Used to create a brand new, empty GUI Tree
	function TECMBindEditor::newMenu( %this ) {

		//Clear the GUI Tree
		ECMGuiTree.clear();

		//Grab the object handle for the root item in the Tree
		%guiRoot = ECMGuiTree.getFirstRootItem();

		//Check VoiceMode, add root item accordingly
		if ($pref::TracerECM::VoiceMode == 1)
			%newGuiId = ECMGuiTree.insertItem( %guiRoot, "TDX NULL CHAT MENU: " @ TracerECM.GetDisplayVoice(TracerECM.PlayerVoice), 0 );
		else if ($pref::TracerECM::VoiceMode == 2)
			%newGuiId = ECMGuiTree.insertItem( %guiRoot, "TDX CANNED CHAT MENU: " @ TracerECM.GetDisplayVoice(TracerECM.PlayerVoice), 0 );
		else
			%newGuiId = ECMGuiTree.insertItem( %guiRoot, "TDX ENHANCED CHAT MENU: " @ TracerECM.GetDisplayVoice(TracerECM.PlayerVoice), 0 );

		ECMGuiTree.dirty = true;

	}


	function ECMCreateBlankCommandObj() {

		%commandObj = new scriptObject(){};

		//echo("\c4CREATING A NEW COMMAND OBJECT" @ %commandObj);

		ECMenuBuild.CommandObjects.add(%commandObj);

		for (%i = 0; %i < $TracerECM::VoiceCount; %i++) {

			%commandObj.chatText[%i] = "";
			%commandObj.chatSound[%i] = "";
			%commandObj.nullChatText[%i] = "";
			%commandObj.nullChatSound[%i] = "";

		}
		%commandObj.isTeam = false;

		TECMBindEditor.addCommand(%commandObj);

	}


//Bind Editor Functions__________________________________________________________________________________
//-----------------------------------------------------------------------------------------------------

	function TECMBindEditor::showTDXMenu( %this, %calledFromWarriorPane ) {

		if ($pref::TracerECM::OperationalMode == 1 && isObject(PlayerListGroup)) 
			%VoiceModeVis = false;
		else
			%VoiceModeVis = true;

		//Set Voice Mode Selection box to appropriate visisblily
		ECMVoiceMode.visible = %VoiceModeVis;

		//Set Text Labels accordingly
		if (!%VoiceModeVis) {

			//Show Text Label
			ECMNullVoiceText.Visible = true;
			
		} else {

			//Hide Text Label
			ECMNullVoiceText.Visible = false;

			//Grab selected voice from Player Prefs
			TracerECM.getPlayerVoice();

		}

		//Save Visibility Mode for future reference
		TracerECM.PlayerVoiceVisMode = %VoiceModeVis;

		//Show Dialog
		Canvas.pushDialog( ECMEditMenu );

	}

	//Excapes Quotes (")
	function ECMfixQuotesForSave(%string) {

		%len = strlen(%string);
		%newString = "";

		for (%i = 0; %i < %len; %i++) {

			%c = getSubStr(%string, %i, 1);

			if (%c $= "\"")
				%c = "\\\"";

			%newString = %newString @ %c;

		}

		return %newString;

	}

//In Game Edit Function________________________________________________________________________________
//-------------------------------------------------------------------------------------------------------

	//A Key should be Bound to this function
	function ECMEditNow(%val) {

		//Display the Editor dialog without the ability to select voice.
		if (%val)
			TECMBindEditor.showTDXMenu();

	}

//_______________________________________________________________________________________________________
//-----------------------------------------------------------------------------------------------------

//Import/Export Commands_______________________________________________________________________________
//-------------------------------------------------------------------------------------------------------

function ECM_ImportMenu() {
	ShellGetLoadFilename( "ECM Import Menu", "scripts/TDX/ECM/Menus/*.cs", "", "ECM_DoImportMenu" );
}
function ECM_DoImportMenu(%filename) {

	//echo("passed menu: " @ %filename);

	//Activate Menu (when given a parameter, this function loads that file instead of the hardcoded one)
	ECMenuBuild.activateMenu("scripts/TDX/ECM/Menus/" @ %filename @ ".cs");

	//Refresh GUI tree with loaded data
	TECMBindEditor.fillMenuTree();
	
	//The tree is now dirty
	ECMGuiTree.dirty = true;
}

function ECM_ExportMenu() {
	ShellGetSaveFilename( "ECM Export Menu", "scripts/TDX/ECM/Menus/*.cs", "", "ECM_DoExportMenu");
}
function ECM_DoExportMenu(%filename) {
	
	TECMBindEditor.doSaveMenu( "scripts/TDX/ECM/Menus/" @ %filename @ ".cs" );
	
}
//_______________________________________________________________________________________________________
//-----------------------------------------------------------------------------------------------------

//Add Item to SimGroup_________________________________________________________________________________
//-------------------------------------------------------------------------------------------------------
TracerECM.Modules.add(TECMBindEditor);

