if (isObject(ECMSoundExplorer))
	ECMSoundExplorer.delete();

new ScriptObject(ECMSoundExplorer) {

	//Class Assignment
	class = ECMSoundExplorer;

	//Module Description
	Version = "3.0";
	DisplayName = "ECM Sound Explorer. 3.0 Renovation";

	ListBuild = false;
	CannedCount = 0;
	
	//This voice index is what the "ALL" voice class will be based on (Male1)
	PrototypeVoice = 4;

};

$TracerECM::DirectoryCount = 0;
$TracerECM::SoundCount = 0;
$TracerECM::DirLabelCount = 0;
$TracerECM::WavLabelCount = 0;



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// S O U N D   L I S T   B U I L D   F U N C T I O N S
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Sound List Prebuild. This takes anywhere from 3 to 20 seconds and is only performed once during startup
function ECMSoundExplorer::BuildSoundList(%this) {

	//First "Directory" is the "All Sounds" directory (#0)qa
	%this.addDirectoryEntry("<ALL SOUNDS>");

	%c = 0;
	for (%file = findFirstFile("*.wav"); %file !$= ""; %file = findNextFile("*.wav")) {
		
		if (getSubStr(%file, 0, 6) $= "audio/") {

			//Truncate redundant "audio/"
			%file = ECMSoundExplorer.TruncateAudio(%file);

			//Find last '/' (To seperate 'directory' and 'wave' strings)
			%a = -1;
			%len = strlen(%file);
			for (%i = 0; %i < %len; %i++) {

				if (getSubStr(%file, %i, 1) $= "/")
					%a = (%i + 1);
			}

			if (%a != -1) {
				%dir = getSubStr(%file, 0, %a);
				%wav = getSubStr(%file, %a, (%len - %a));

				%dirNum = %this.addDirectoryEntry(%dir);
				%this.addSoundEntry(%wav, %dirNum);
			}
		}

	}

	%this.ListBuild = true;

}

function ECMSoundExplorer::addDirectoryEntry(%this, %dir) {

	//Search Directory Listing for existing entry
	for (%i = 0; %i < $TracerECM::DirectoryCount; %i++)
		if ($TracerECM::DirectoryList[%i] $= %dir)
			return %i;

	//Create new entry
	$TracerECM::DirectoryList[$TracerECM::DirectoryCount] = %dir;
	
	//Check if this is a voice directory
	for (%i = 0; %i < $TracerECM::VoiceCount; %i++) {

		//If it is a voice directory, save it in an index for later use
		if (("voice/" @ $TracerECM::VoiceDirs[%i] @ "/") $= %dir) {
		
			$TracerECM::VoiceDirIndex[%i] = $TracerECM::DirectoryCount;
			
			//If this voice directory is the PrototpyeVoice, store it for later use
			if (%i == ECMSoundExplorer.PrototypeVoice)
				$TracerECM::VoiceDirIndex[-1] = $TracerECM::DirectoryCount;
			
		}
			
	}
	$TracerECM::DirectoryLabel[$TracerECM::DirectoryCount] = %this.GetDirResourceLabel(%dir);
	$TracerECM::DirectoryCount++;
	return $TracerECM::DirectoryCount - 1;

}

function ECMSoundExplorer::addSoundEntry(%this, %wav, %dir) {

	//Don't Add sounds that start with '_' (Hidden Sound)
	if (getSubStr(%wav, 0, 1) $= "_")
		return;

	//Create new entry
	$TracerECM::SoundList[$TracerECM::SoundCount] = %wav;
	$TracerECM::SoundDirList[$TracerECM::SoundCount] = %dir;
	$TracerECM::SoundLabel[$TracerECM::SoundCount] = %this.GetWavResourceLabel($TracerECM::DirectoryList[%dir] @ %wav);
	$TracerECM::SoundCount++;
	
	//If this sound is part of the prototype voice directory, then store this sound index in the prototype ALL voice...
	//if (%dir == ECMSoundExplorer.PrototypeVoiceDir) {
		
	//	$TracerECM::ALLSoundList[$TracerECM::ALLSoundCount] = %wav;
	//	$TracerECM::ALLSoundLabel[$TracerECM::SoundCount] = %this.GetWavResourceLabel($TracerECM::DirectoryList[%dir] @ %wav);
	//	$TracerECM::ALLSoundCount++;
	
	//}
	
	return $TracerECM::SoundCount - 1;

}

//TEST/DIAGNOSTIC FUNCTION: CONSOLE DUMP
function ECMDumpSoundTable() {

	echo("\c4Dumping Contents of the Sound Entry Table");

	for (%i = 0; %i < $TracerECM::SoundCount; %i++) {

		echo("\c3#" @ %i @ " - " @ $TracerECM::SoundList[%i] @ " in (" @ $TracerECM::DirectoryList[$TracerECM::SoundDirList[%i]] @ ")");

	}

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// R E S O U R C E   L A B E L I N G   F U N C T I O N S
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//This function is called by the resource label listing script (ie, the "saved" file)
function ECMSoundExplorer::AddDirResourceLabel(%this, %DirResource, %label) {

	//$TracerECM::DirLabelResource[$TracerECM::DirLabelCount] = %DirResource;
	$TracerECM::DirLabelText[%DirResource] = %label;
	//$TracerECM::DirLabelCount++;

}

function ECMSoundExplorer::AddWavResourceLabel(%this, %WavResource, %label) {

	//$TracerECM::WavLabelResource[$TracerECM::WavLabelCount] = %WavResource;
	$TracerECM::WavLabelText[%WavResource] = %label;
	//$TracerECM::WavLabelCount++;

}

//This function will return a directory label 
function ECMSoundExplorer::GetDirResourceLabel(%this, %DirResource) {

	//for (%i = 0; %i < $TracerECM::DirLabelCount; %i++) {
	
	//	if ($TracerECM::DirLabelResource[%i] $= %DirResource)
			return $TracerECM::DirLabelText[%DirResource];
	
	//}
	
	//return "";

}

//This function will return a wav label
function ECMSoundExplorer::GetWavResourceLabel(%this, %WavResource) {

	//for (%i = 0; %i < $TracerECM::WavLabelCount; %i++) {
	
	//	if ($TracerECM::WavLabelResource[%i] $= %WavResource)
			return $TracerECM::WavLabelText[%WavResource];
	
	//}
	
	//return "";

}

//This function, called by the voicepack driver, will register a label for the wav entry of the voicepack item with the voicepack text
function ECMSoundExplorer::AddVPWavResourceLabel(%this, %WavResource, %label) {

	//Check to see there isn't already a label for this
	if (%this.GetWavResourceLabel(%WavResource) $= "") {
	
		//If not, add it
		%this.AddWavResourceLabel(%WavResource, %label);
	
	}

}

//This function is for DEVELOPMENT purposes only:
//It will take all the text from binds in the currently loaded menu and apply 
//them as labels to the sound in the same bind. Then save it all to the 
//ResourceLabels.cs file
//NOTE: This whole thing will fail if you do it AFTER the browser has loaded
//      or BEFORE the menu is loaded
//ECMSoundExplorer.ExtractResourceLabelsFromECMenu(); into the console
function ECMSoundExplorer::ExtractResourceLabelsFromECMenu(%this) {

	MessagePopup( "LABEL LIST BUILD", "Traversing ECMenu and adding labels based on nullvoice command binds");
	Canvas.repaint(); //Make Sure this message is displayed before the client lockup from processing
	%this.writeResourceDataNode(ECMenuBuild.RootMenu);
	CloseMessagePopup();
	
	//Cause list to load
	MessagePopup( "SOUNDLIST BUILD", "Scanning client for Sound Files\nThis could take a few moments");
	Canvas.repaint(); //Make Sure this message is displayed before the client lockup from processing
	%this.BuildSoundList();
	CloseMessagePopup();
	
	//Save results
	%this.SaveResourceLabels();

}

//Self calling function that traverses the GUI Tree and saves all items to a file

function ECMSoundExplorer::writeResourceDataNode( %this, %item) {

	for (%a = 0; %a < %item.optionCount; %a++) {

		%temp = %item.option[%a];

		//Parse a few items out of the text
		%key = getSubStr( %temp, 0, strpos(%temp, ":") );
		%text = getSubStr(%temp, (strpos(%temp, ":")+2), (strlen(%temp) - (strpos(%temp, ":")+2)));
		%command = %item.command[%a];

		//Find out if this is a menu item or bind (command) item 
		if ( %item.isMenu[%a] == 1 ) {

			//Grab the first child item of this submenu
			%child = %item.command[%a];
			
			if (isObject(%child))
				%this.writeResourceDataNode(%child);

		} else {

			//Parse out Data for this bind (command) item
			%commandObj = %command;

			

			// Extract Data from the Nullvoice item (since they are usually the most complete)
			for (%i = 0; %i < 14; %i++) {
			
				//Grab the items we want
				%text = %commandObj.nullChatText[%i];
				%sound = %commandObj.nullChatSound[%i];
				
				//Create a label entry for this (if not blank)
				if ((%sound !$= "") && (%text !$= ""))
					%this.AddWavResourceLabel(%sound @ ".wav", %text);
				
			}

		}

	}

}



//This function will save currently loaded labels to the load file (no re-load needed)
function ECMSoundExplorer::SaveResourceLabels() {

	//Create filesytem handle object
	//Create a new file object so we can write this shit
	%myFSO = new fileObject();

	//Open file for write
	%myFSO.openForWrite( "scripts/TDX/ECM/ResourceLabels.cs" );
	
	//Print out a little header so people know what the hell this file is
	%myFSO.writeLine( "//----------------------------------------------------------------------------\\\\");
	%myFSO.writeLine( "//                                                                            \\\\");
	%myFSO.writeLine( "//        TracerDX's Enhanced Chat - Resource Label Definition File           \\\\");
	%myFSO.writeLine( "//                                                                            \\\\");
	%myFSO.writeLine( "//               Written by TracerDX (tracerdx@tracerdx.com)                  \\\\");
	%myFSO.writeLine( "//                     http://tdxscripts.tracerdx.com                         \\\\");
	%myFSO.writeLine( "//                                                                            \\\\");
	%myFSO.writeLine( "//----------------------------------------------------------------------------\\\\");
	%myFSO.writeLine( "");
	%myFSO.writeLine( "");
	
	//Save Directory Labels
	%myFSO.writeLine( "//---[DIRECTORY LABELS]-----------------------------------------------------------");
	for (%i = 0; %i < $TracerECM::DirectoryCount; %i++) {
	
		if ($TracerECM::DirectoryLabel[%i] !$= "")
			%myFSO.writeLine( "ECMSoundExplorer.AddDirResourceLabel(\"" @ $TracerECM::DirectoryList[%i] @ "\", \"" @ ECMFixEscapeSequences($TracerECM::DirectoryLabel[%i]) @ "\");");
	
	}

	//Spacer
	%myFSO.writeLine( "");
	%myFSO.writeLine( "");

	//Save Wav Labels
	%myFSO.writeLine( "//---[SOUND FILE LABELS]----------------------------------------------------------");
	for (%i = 0; %i < $TracerECM::SoundCount; %i++) {
		
		if ($TracerECM::SoundLabel[%i] !$= "")
			%myFSO.writeLine( "ECMSoundExplorer.AddWavResourceLabel(\"" @ $TracerECM::DirectoryList[$TracerECM::SoundDirList[%i]] @ $TracerECM::SoundList[%i] @ "\", \"" @ ECMFixEscapeSequences($TracerECM::SoundLabel[%i]) @ "\");");
		
	}
	
	//Close File
	%myFSO.close();
	
	//Destroy File System Object
	%myFSO.delete();
	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// S O U N D   L I S T   S U P P O R T   A N D   L O O K U P   F U N C T I O N S
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Will take the full path of a sound file and splice the data into
//ECMSoundExplorer.CurrentDir = (dir#)
//ECMSoundExplorer.CurrentSound = (wav#)
function ECMSoundExplorer::SpliceSound(%this, %soundfile) {

	%len = strlen(%soundfile);
	for (%i = 0; %i < %len; %i++) {

		if (getSubStr(%soundfile, %i, 1) $= "/")
			%a = (%i + 1);
	}

	%dir = getSubStr(%soundfile, 0, %a);
	%wav = getSubStr(%soundfile, %a, (%len - %a));
	
	%this.CurrentDir = %this.FindDirectoryNum(%dir);
	%this.CurrentWav = %this.FindSoundNum(%this.CurrentDir, %wav);
	
}

//Returns Directory Entry # or -1 if not found
function ECMSoundExplorer::FindDirectoryNum(%this, %dir) {

	//Search Directory Listing for existing entry
	for (%i = 0; %i < $TracerECM::DirectoryCount; %i++)
		if ($TracerECM::DirectoryList[%i] $= %dir)
			return %i;

	return -1;

}

//Returns Sound Entry # or -1 if not found
function ECMSoundExplorer::FindSoundNum(%this, %dir, %snd) {

	//Search Directory Listing for existing entry
	for (%i = 0; %i < $TracerECM::SoundCount; %i++)
		if (($TracerECM::SoundDirList[%i] == %dir) && ($TracerECM::SoundList[%i] $= %snd))
			return %i;

	return -1;

}


//Lops off the audio/ in front of all sounds
function ECMSoundExplorer::TruncateAudio(%this, %sound) {

	%len = strlen(%sound);
	return getSubStr(%sound, 6, (%len - 6));

}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// S O U N D   E X P L O R E R   G U I   F U N C T I O N S
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Entry point function. Called to present Sound Explorer to user.
function ECMSoundExplorer::GetSound(%this, %callback, %oldSound, %notNullvoice, %voiceNum) {

	//Set Environemnt values
	%this.Callback = %callback;
	%this.notNullvoice = %notNullvoice;
	%this.voiceNum = %voiceNum;

	//Show Explorer GUI (Will cause some initialization)
	Canvas.pushDialog(ECMSoundExplorerGui);
	
	//Hide the Test Sound button if voice = -1 (ALL VOICES) - [B301-002] AND %notNullvoice == true
	ECMSoundExplorerTestSoundBtn.setVisible(!((%voiceNum == -1) && %notNullvoice));
	
	//same for the Set Label Button
	ECMSoundExplorerSetLabelBtn.setVisible(!((%voiceNum == -1) && %notNullvoice));

	//Prep the old sound file for selection
	ECMSoundExplorer.SpliceSound(%oldSound);

	//Update Selection to ECMSoundExplorer.CurrentDir/ECMSoundExplorer.CurrentSound
	%this.UpdateSelected(ECMSoundExplorer.CurrentDir, ECMSoundExplorer.CurrentWav);

}

function ECMSoundExplorer::UpdateSelected(%this, %dir, %wav) {

	if ((%dir != -1) && (!%this.notNullvoice)) {
		ECMGroupList.setSelected(%dir);
		ECMSoundItemList.fillList();
	}

}


//---[ECMSoundFilePath]--------------------------------------------------------------------------------------------
function ECMSoundFilePath::UpdateFilePathDisplay(%this) {

	if (ECMSoundExplorer.notNullvoice) {
		%dir = $TracerECM::VoiceDirIndex[ECMSoundExplorer.VoiceNum];
	} else {
		%dir = ECMGroupList.getSelected();
	}
	
	%wav = ECMSoundItemList.getSelectedId();
	
	if (%dir == 0) //<ALL SOUNDS>
		%dir = $TracerECM::SoundDirList[%wav];
	
	if ((ECMSoundExplorer.VoiceNum == -1) && (ECMSoundExplorer.notNullvoice))
		%text = "/base/audio/voice/*/" @ $TracerECM::SoundList[%wav];
	else
		%text = "/base/audio/" @ $TracerECM::DirectoryList[%dir] @ $TracerECM::SoundList[%wav];

	ECMSoundFilePath.setText(%text);

}


//---[ECMSoundExplorerGui]-----------------------------------------------------------------------------------------
function ECMSoundExplorerGui::onWake(%this) {

	if (!ECMSoundExplorer.ListBuild) {
		MessagePopup( "SOUNDLIST BUILD", "Scanning client for Sound Files\nThis could take a few moments");
		Canvas.repaint(); //Make Sure this message is displayed before the client lockup from processing
		ECMSoundExplorer.BuildSoundList();
		CloseMessagePopup();
	}

	//Set Title (So selection mode is obvious)
	if (ECMSoundExplorer.notNullvoice) {
	
		if (ECMSoundExplorer.voiceNum == -1) { //ALL Mode
			
			ECMSoundExplorerPane.setTitle("ECM SOUND EXPLORER [ALL VOICES]");
		
		} else {
	
			//Grab directoy num indexed by voicenum
			%dir = $TracerECM::VoiceDirIndex[ECMSoundExplorer.voiceNum];

			//Use "pretty" label if applicable
			if ($TracerECM::DirectoryLabel[%dir] $= "") {
				ECMSoundExplorerPane.setTitle("ECM SOUND EXPLORER [" @ $TracerECM::DirectoryList[%dir] @ "]");
			} else {
				ECMSoundExplorerPane.setTitle("ECM SOUND EXPLORER [" @ $TracerECM::DirectoryLabel[%dir] @ "]");
			}
		
		}
		
	} else {
		ECMSoundExplorerPane.setTitle("ECM SOUND EXPLORER [NULLVOICE]");
	}

	//Fill Directory List
	ECMGroupList.fillList();
	ECMSoundItemList.fillList();

}
//-----------------------------------------------------------------------------------------------------------------

//---[ECMGroupList]------------------------------------------------------------------------------------------------
function ECMGroupList::fillList( %this ) {

	//Clear List
	%this.clear();

	if (!ECMSoundExplorer.notNullvoice) {
	
		%this.setVisible(true);
		ECMGroupListLabel.setVisible(true);

		for (%i = 0; %i < $TracerECM::DirectoryCount; %i++) {
			if ($TracerECM::DirectoryLabel[%i] $= "") {
				%this.add("[" @ $TracerECM::DirectoryList[%i] @ "]", %i);
			} else {
				%this.add($TracerECM::DirectoryLabel[%i], %i);
			}
		}

	} else {
		
		%this.setVisible(false);
		ECMGroupListLabel.setVisible(false);
		//%this.add($TracerECM::DirectoryLabel[$TracerECM::VoiceDirIndex[ECMSoundExplorer.voiceNum]], 1);

	}

}

function ECMGroupList::onSelect(%this) {

	//Load in the sounds for this directory
	ECMSoundItemList.fillList();
	
	//Select the first Item if nothing is selected
	if (ECMSoundItemList.getSelectedId() == -1)
		ECMSoundItemList.setSelectedRow(0);

}
//-----------------------------------------------------------------------------------------------------------------

//---[ECMSoundItemList]--------------------------------------------------------------------------------------------
function ECMSoundItemList::fillList(%this) {

	%this.clear();

	if (!ECMSoundExplorer.notNullvoice) {
		%dir = ECMGroupList.getSelected();
	} else {
		%dir = $TracerECM::VoiceDirIndex[ECMSoundExplorer.voiceNum];
	}

	%curRow = 0;

	for (%i = 0; %i < $TracerECM::SoundCount; %i++) {

		if ((%dir == 0) || ($TracerECM::SoundDirList[%i] == %dir)) {

			if ($TracerECM::SoundLabel[%i] $= "") {
				%this.addRow(%i, "[" @ $TracerECM::SoundList[%i] @ "]" TAB $TracerECM::SoundList[%i]);
			} else {
				%this.addRow(%i, $TracerECM::SoundLabel[%i] TAB $TracerECM::SoundList[%i]);
			}

			if (%i == ECMSoundExplorer.CurrentWav)
				%this.setSelectedRow(%curRow);

			%curRow++;

		}

	}
		
	

}

function ECMSoundItemList::onSelect(%this) {

	//Update file path to selected item
	ECMSoundFilePath.UpdateFilePathDisplay();

}

//Right click event for Right click menu
function ECMSoundItemList::onRightMouseDown( %this, %a1, %item, %pos ) {

	ECMSoundItemList.setSelectedRow( %item );
	ECMSoundExplorerActionPopup.awaken( %pos );

}


//-----------------------------------------------------------------------------------------------------------------

//---[TEST Button]-------------------------------------------------------------------------------------------------
function ECMSoundExplorer::TestSelSound( %this ) {

	//If in ALL VOICES mode, do not test sound - [B301-009] EXCEPT if in Nullvoice selection mode
	if ((ECMSoundExplorer.voiceNum == -1) && (%this.notNullvoice))
		return;

	%wav = ECMSoundItemList.getSelectedId();
	%dir = $TracerECM::SoundDirList[%wav];
	%sound = ($TracerECM::DirectoryList[%dir] @ $TracerECM::SoundList[%wav]);

	alxPlay( alxCreateSource( AudioGui, %sound ) );

}
//-----------------------------------------------------------------------------------------------------------------

//---[SELECT Button]-----------------------------------------------------------------------------------------------
function ECMSoundExplorer::selectSound( %this ) {

	%wav = ECMSoundItemList.getSelectedId();
	%dir = $TracerECM::SoundDirList[%wav];
	%sound = ($TracerECM::DirectoryList[%dir] @ $TracerECM::SoundList[%wav]);

	//Hide Explorer
	Canvas.popDialog(ECMSoundExplorerGui);

	//Call callback function with user's selected sound
	if (%this.notNullvoice) {
		call(%this.Callback, $TracerECM::SoundList[%wav]);
	} else {
		call(%this.Callback, %sound);
	}

}

//---[ECMSoundExplorerLabelGui]------------------------------------------------------------------------------------
function ECMSoundExplorerLabelGui::onWake( %this ) {

	//If in ALL VOICES mode, do not change labels 
	//Technically, it is impossible to get to this point in this mode, but very bad errors prompted me to make damned sure it doesn't happen.
	if ((ECMSoundExplorer.voiceNum == -1) && (ECMSoundExplorer.notNullvoice)) {
		Canvas.popDialog(ECMSoundExplorerLabelGui);
		return;
	}


	if (ECMSoundExplorer.notNullvoice) {
		%dir = $TracerECM::VoiceDirIndex[ECMSoundExplorer.VoiceNum];
	} else {
		%dir = ECMGroupList.getSelected();
	}
	%wav = ECMSoundItemList.getSelectedId();
	%text = "/base/audio/" @ $TracerECM::DirectoryList[%dir] @ $TracerECM::SoundList[%wav];
	%label = $TracerECM::SoundLabel[%wav];
	
	
	ECMEditLabelFile.setText(%text);
	ECMEditLabel.setText(%label);

}

//---[OK Button]---------------------------------------------------------------------------------------------------
function ECMSoundExplorer::doSetSelLabel( %this ) {

	//If in ALL VOICES mode, do not change labels 
	//Technically, it is impossible to get to this point in this mode, but very bad errors prompted me to make damned sure it doesn't happen.
	if ((ECMSoundExplorer.voiceNum == -1) && (%this.notNullvoice))
		return;


	//Grab Current wav ID
	%wav = ECMSoundItemList.getSelectedId();
	
	//This will keep the selection from moving (annoyingly)
	ECMSoundExplorer.CurrentWav = %wav;
	
	//Set Label
	$TracerECM::SoundLabel[%wav] = ECMEditLabel.getValue();
	
	//Save labels to file
	ECMSoundExplorer.SaveResourceLabels();

	//Refresh sound file list display (to show label change)
	ECMSoundItemList.fillList();
	
	//Hide label edit dialog
	Canvas.popDialog(ECMSoundExplorerLabelGui);
}

//---[ECMSoundExplorerActionPopup]---------------------------------------------------------------------------------
function ECMSoundExplorerActionPopup::reset( %this ) {

	//Clear the entry counter
	%this.numEntries = 0;

	//Cancels the onCancel event caused by %this.forceOnAction();
	%this.forceClose();

	//Remove the menu from visible GUI parent object
	Canvas.popDialog( ECMSoundExplorerActionDlg );

}

//Called during menu construction to add an option to right click menu
function ECMSoundExplorerActionPopup::addEntry( %this, %text, %id ) {

	

	//Increment Item Count Counter
	%this.numEntries++;

	//Add the item to the menu
	%this.add( %text, %id );

}

//Called to show the right Click Menu...
function ECMSoundExplorerActionPopup::awaken( %this, %pos ) {

	//Initialize GUI Panel to mouse position and clear items
	%this.position = %pos;
	%this.clear();

	%label = $TracerECM::SoundLabel[ECMSoundItemList.getSelectedId()];
	%wav = $TracerECM::SoundList[ECMSoundItemList.getSelectedId()];
	//Header
	%this.addEntry( "\c2 -[ " @ ((%label !$= "") ? %label : %wav) @ " ]-", 1 );

	//Test Sound - Do not show in ALL VOICES mode
	if ((ECMSoundExplorer.voiceNum != -1) && (ECMSoundExplorer.notNullvoice))
		%this.addEntry( "    Test Sound", 1 );

	//Select Sound
	%this.addEntry( "    Select Sound",2 );
	
	//Change Label - Do not show in ALL VOICES mode
	if ((ECMSoundExplorer.voiceNum != -1) && (ECMSoundExplorer.notNullvoice))
		%this.addEntry( "    Set Label", 3 );

	//Show the newly constructed right click menu
	Canvas.pushDialog( ECMSoundExplorerActionDlg ); 

	//Causes %this.onCancel to be called if user clicks outside of the menu
	%this.forceOnAction();

}

//Called when the user selects and item
function ECMSoundExplorerActionPopup::onSelect( %this, %id, %text ) {
	
	if (%id == 0) return;

	//Call Menu Removeal Function
	%this.reset();
	
	switch ( %id ) {
		case 1: // Test Sound
			ECMSoundExplorer.TestSelSound();
		case 2: // Select Sound
			ECMSoundExplorer.selectSound();
		case 3: // Set Label
			Canvas.pushDialog(ECMSoundExplorerLabelGui);
	}

}

//Called if user clicks outside of menu (caused by %this.forceOnAction();)
function ECMSoundExplorerActionPopup::onCancel( %this ) {

	//Call Menu Removeal Function
	%this.reset();

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// C A N N E D   C H A T   I T E M   L I S T   B U I L D E R   F U N C T I O N S
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ECMSoundExplorer::addCannedItem( %this, %cannedChatItem, %label) {

	%this.CannedList[%this.CannedCount] = %label;
	%this.CannedItems[%this.CannedCount] = %cannedChatItem;

	%this.CannedCount++;

}

function ECMSoundExplorer::addCIHeader( %this, %label) {

	%this.CannedList[%this.CannedCount] = %label;
	%this.CannedItems[%this.CannedCount] = false;

	%this.CannedCount++;

}













//Load Resource Labels...
if (isFile("scripts/TDX/ECM/ResourceLabels.cs")) {
	
	//Delete .dso
	deleteFile("scripts/TDX/ECM/ResourceLabels.cs.dso");
	//Load items
	exec("scripts/TDX/ECM/ResourceLabels.cs");
	
} else {

	//Create ResourceLabels.cs file from DefaultResourceLabels.cs if not present
	exec("scripts/TDX/ECM/DefaultResourceLabels.cs");
	
	//Need to build SoundList to save this.
	ECMSoundExplorer.BuildSoundList();
	
	//Save the list to the normal file (ResourceLabels.cs)
	ECMSoundExplorer.SaveResourceLabels();
}
