




//Create Driver Object
new ScriptObject(VPD6ECMDriver) {
	class = "VPD6ECMDriver";
	Name = "D6 Voicepack Protocol";
	OAuthor = "Govt.Cheese[D6]";
	DAuthor = "TracerDX";
	Version = "1.0";
	ConverterMode = false;
	
};

//Register This Driver with ECM, and store ID
VPD6ECMDriver.ID = ECMVPConversion.RegisterProtocol(VPD6ECMDriver);


//This entry point function will be called on the object when ECM wishes to have this driver
//convert is specific VoicePack file in this protocol. The VPID is used to create the new
//file in scripts/voicepacks/converted.[ID].cs It is the driver's job to attempt to salvage
//what information it can from the voicepack and write a new voicepack file.
//NOTE: It is the converter's job to make sure to warn the user before over-writing existing
//      converted packs.
function VPD6ECMDriver::ConvertVoicePack(%this, %VPID, %file) {

	echo("\c6Checking " @ "scripts/VoicePacks/coverted.d6." @ %VPID @ "0.cs");

	if (isFile("scripts/VoicePacks/coverted.d6." @ %VPID @ "0.cs")) {
	
		echo("\c4Warning! File Exists!");
	
		MessageBoxOKCancel("WARNING", "It appears you already have this voicepack converted. Do you wish to over-write your current conversion with a new one?", "VPD6ECMDriver.ReallyConvertVoicePack(\"" @ %VPID @ "\", \"" @ VPD6FixEscapeSequences(%file) @ "\");");
	
	} else {
	
		echo("Converting now");
	
		%this.ReallyConvertVoicePack(%VPID, %file);
	
	}

}
function VPD6ECMDriver::ReallyConvertVoicePack(%this, %VPID, %file) {
	//Trigger "Converter flag" to intercept voicepack definition function calls
	%this.ConverterMode = 1;
	
	//Pass 1: Execute Voicepack file with this driver in Conversion mode 1 to grab "canned chat items" and store in sound table
	echo("\c6Pass 1: Extracting Sound Handles...");
	exec(%file);
	
	//Create new Voicepack file
	new FileObject(defFile);
	
	//Set Enumeration variable
	$VPD6Enum = 0;
	$VPD6Indent = "";
	$VPD6VPID = %VPID;
	
	//Pass 2: Execute Voicepack file with this driver in Conversion mode 2 to grab menu structure
	%this.ConverterMode = 2;
	echo("\c6Pass 2: Extracting Voicepack Menu...");
	exec(%file);
	
	//Finish up file and destroy handle
	defFile.close();
	defFile.delete();
	
	//Return to normal operation...
	%this.ConverterMode = false;
	
}

//This function will transfer the root items of the newly converted voicepack to voicepack entry point items on the root ECMenu
function VPD6ECMDriver::TransferKeys(%this) {

	for (%i = 0; %i < $VPD6Enum; %i++) {
	
		%commandObj = new scriptObject(){};
		
		ECMenuBuild.CommandObjects.add(%commandObj);

		for (%i = 0; %i < $TracerECM::VoiceCount; %i++) {

			%commandObj.chatText[%i] = "";
			%commandObj.chatSound[%i] = "";
			%commandObj.nullChatText[%i] = "";
			%commandObj.nullChatSound[%i] = "";

		}
		%commandObj.isTeam = false;
		%commandObj.startVoicePack = ($VPD6VPID @ %i);
		
		//Select menu root
		ECMGuiTree.selectItem(ECMGuiTree.rootMenuItem);
		
		//Insert into tree at selection (root)
		TECMdoNewCommand( %this.TransferKey[%i], %this.TransferLabel[%i], %commandObj );
		
		//Make sure menu gets saved
		ECMGuiTree.dirty = true;
	
	}

}

//This package is designed to over-ride the commands called by the D6 voicepack definition script
//It will use this information to construct a new file for VPECM
//deactivatePackage(VPD6ECMDriverConverter);
package VPD6ECMDriverConverter {

	function startChatMenu( %heading ) {

		if (!VPD6ECMDriver.ConverterMode) {
			parent::startChatMenu( %heading );
			return;
		}

		if (VPD6ECMDriver.ConverterMode == 2) {
		
			%key = firstWord(%heading);
			%text = restWords(%heading);
		
			//If this is the first startChatMenu(), then it is the start of a whole "Voicepack" for VPD6ECMDriver
			if ($VPD6Indent $= "") {
			
				defFile.openForWrite("scripts/VoicePacks/coverted.d6." @ $VPD6VPID @ $VPD6Enum @ ".cs");
				
				VPD6ECMDriver.TransferKey[$VPD6Enum] = %key;
				VPD6ECMDriver.TransferLabel[$VPD6Enum] = %text;
				
				//Write Header Directives
				defFile.writeLine("//#VOICEPACK");
				defFile.writeLine("//#ID " @ $VPD6VPID @ $VPD6Enum);
				defFile.writeLine("//#NAME " @ %text);
				defFile.writeLine("//#VERSION ?");
				defFile.writeLine("//#DEFAULTKEY " @ %key);
				defFile.writeLine("//#AUTHOR ECM Conversion Util");
				defFile.writeLine("//#DESCRIPTION This voicepack was converted from the D6 protocol. It did not support a description data element, and therefore, this one was included instead. There may be a hand converted version at tdxscripts.tracerdx.com");

				//Write "Pretty" File Header
				defFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
				defFile.writeLine( "//                                                                            \\\\" );
				defFile.writeLine( "//               ECMVoicePack - Voice Package Definition File                 \\\\" );
				defFile.writeLine( "//                                                                            \\\\" );
				defFile.writeLine( "//                 This file is a conversion written by ECM                   \\\\" );
				defFile.writeLine( "//                           Conversion Utility                               \\\\" );
				defFile.writeLine( "//                      http://www.planettracerdx.com                         \\\\" );
				defFile.writeLine( "//                                                                            \\\\" );
				defFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
				defFile.writeLine( " " );
			
			
			} else {
			
			

				//Some characters, namely Quotes, fuck things up since their escape sequences are lost... this adds it back on
				%text = VPD6FixEscapeSequences(%text);

				echo("\n" @ $VPD6Indent @ "startVMenu( \"" @ %key @ ": " @ %text @ "\" );");


				defFile.writeLine( $VPD6Indent );
				defFile.writeLine( $VPD6Indent @ "startVMenu( \"" @ %key @ ": " @ %text @ "\" );" );
				
			}


			$VPD6Indent = $VPD6Indent @ "\t";

		}

	}

	function addChat( %heading, %CannedChatItem ) {

		if (!VPD6ECMDriver.ConverterMode) {
			parent::addChat( %heading, %CannedChatItem );
			return;
		}

		if (VPD6ECMDriver.ConverterMode == 2) {

			%key = firstWord(%heading);
			%htext = restWords(%heading);
			%text = $VPD6ChatTable::Text[%CannedChatItem];
			%sound = $VPD6ChatTable::Sound[%CannedChatItem];

			//Some characters, namely Quotes, fuck things up since their escape sequences are lost... this adds it back on
			%text = VPD6FixEscapeSequences(%text);
			%htext = VPD6FixEscapeSequences(%htext);
			
			if ($VPD6ChatTable::IsTeamChat[%CannedChatItem])
				%isTeamChat = "true";
			else
				%isTeamChat = "false";

			//Truncate K in front of sound
			%sound = getSubStr(%sound, 1, strlen(%sound));

			echo($VPD6Indent @ "addVItem( \"" @ %key @ ": " @ %htext @ "\", \"" @ %text @ "\", \"" @ %sound @ "\", " @ %isTeamChat @ ")");

			//addVPD6Item(%keyDesc, %text, %sound, %isTeamChat)
			defFile.writeLine( $VPD6Indent @ "addVItem( \"" @ %key @ ": " @ %htext @ "\", \"" @ %text @ "\", \"" @ %sound @ "\", " @ %isTeamChat @ ");" );

		}

	}

	function endChatMenu() {

		if (!VPD6ECMDriver.ConverterMode) {
			
			parent::endChatMenu();
			return;
		}
		
		if (VPD6ECMDriver.ConverterMode == 2) {

			$VPD6Indent = getSubStr($VPD6Indent, 0, (strlen($VPD6Indent) - 1));

			
			
			//if the indent is gone, then this is the end of a root item, which is the end of a "Voicepack".
			if ($VPD6Indent $= "") {
			
				
				//The next startChatMenu() will cause the next file to be opened for write
				defFile.close();
				
				//Increment Enumeration Variable
				$VPD6Enum++;
			
			} else {

				echo($VPD6Indent @ "endVMenu();");
				
				defFile.writeLine( $VPD6Indent @ "endVMenu();" );
				defFile.writeLine( $VPD6Indent );
				
			}

		}
		

	}

	function installChatItem( %command, %text, %audioFile, %animCel, %teamOnly, %defaultkeys, %play3D ) {

		if (!VPD6ECMDriver.ConverterMode) {
			parent::installChatItem( %command, %text, %audioFile, %animCel, %teamOnly, %defaultkeys, %play3D );
			return;
		}

		if (VPD6ECMDriver.ConverterMode == 1) {
			$VPD6ChatTable::Text[%command] = %text;
			$VPD6ChatTable::Sound[%command] = %audioFile;
			$VPD6ChatTable::IsTeamChat[%command] = %teamOnly;
		}

	}


	//This is where I intercept old VoicePack registration attempts. This effectively disables the old VoicePack.vl2
	//I also use the intercepted information to try and convert the old voicepack.
	function VoicePackRegisterName( %packID ) {

		%filename = "scripts/" @ %packID @ "VoicePack.cs";
			
		echo("\c6d6 Voicepack detected: " @ %packID @ " - File: " @ %filename);
			
		ECMVPConversion.RegisterOldPack(%packID, %filename, VPD6ECMDriver.ID);

	}

};
activatePackage(VPD6ECMDriverConverter);


//This function will fix characters that will be "executed" in a string by makeing them the escape sequence string instead
function VPD6FixEscapeSequences(%string) {

	%newString = "";

	for (%i = 0; %i < strlen(%string); %i++) {

		%ch = getSubStr(%string, %i, 1);
		switch$ (%ch) {
		case "\"":
			%ch = "\\\"";
		case "\\":
			%ch = "\\\\";
		case "\c0":
			%ch = "\\c0";
		case "\c1":
			%ch = "\\c1";
		case "\c2":
			%ch = "\\c2";
		case "\c3":
			%ch = "\\c3";
		case "\c4":
			%ch = "\\c4";
		case "\c5":
			%ch = "\\c5";
		case "\c6":
			%ch = "\\c6";
		case "\c7":
			%ch = "\\c7";
		case "\c8":
			%ch = "\\c8";
		case "\c9":
			%ch = "\\c9";
		case "\cr":
			%ch = "\\cr";
		}

		%newString = %newString @ %ch;

		}
	
	return %newString;

}